unit Mabfsave;

interface

uses
   SysUtils, WinTypes, WinProcs, Classes, Graphics, Forms, MabSave, IniFiles;

type
   TFormProperty = (fpColor, fpFont, fpPosition, fpSize, fpWindowState);
   TFormProperties = set of TFormProperty;
   TMabFormSave = class(TMabSave)
   private
      { Private declarations }
      FRemember: TFormProperties;
   protected
      { Protected declarations }
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;

      procedure Save;
      procedure Load;
   published
      { Published declarations }
      property FileName;
      property Remember: TFormProperties read FRemember write FRemember;
   end;

implementation

constructor TMabFormSave.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FRemember := [fpPosition, fpSize, fpWindowState];
end;

procedure TMabFormSave.Save;
var
   PName, PMesg: PChar;
   Section: string;
begin
   try
      if IniFile = nil then
         raise Exception.Create('Filename must be have a value');
      if not (Owner is TForm) then
         raise Exception.Create('Owner must be of type TForm');

      with Owner as TForm do
      begin
         if fpColor in Remember then
         begin
            Section := ClassName + '.Color';

            IniFile.WriteString(Section, 'Color', ColorToString(Color));
         end;

         if fpFont in Remember then
         begin
            Section := ClassName + '.Font';

            IniFile.WriteString (Section, 'Name',   Font.Name);
            IniFile.WriteInteger(Section, 'Size',   Font.Size);
            IniFile.WriteString (Section, 'Color',  ColorToString(Font.Color));
            IniFile.WriteBool   (Section, 'Italic', (fsItalic in Font.Style));
            IniFile.WriteBool   (Section, 'Bold',   (fsBold in Font.Style));
            IniFile.WriteBool   (Section, 'Underline', (fsUnderline in Font.Style));
            IniFile.WriteBool   (Section, 'Strikeout', (fsStrikeout in Font.Style));
         end;

         if fpWindowState in Remember then
         begin
            Section := ClassName + '.Window';

            if (WindowState = wsMinimized) or IsIconic(Application.Handle) then
               IniFile.WriteBool(Section, 'Minimized', True)
            else if WindowState = wsMaximized then
            begin
               IniFile.WriteBool(Section, 'Minimized', False);
               IniFile.WriteBool(Section, 'Maximized', True)
            end
            else if WindowState = wsNormal then
            begin
               IniFile.WriteBool(Section, 'Minimized', False);
               IniFile.WriteBool(Section, 'Maximized', False);
            end;
         end;

         WindowState := wsNormal;
         if fpPosition in Remember then
         begin
            Section := ClassName + '.Window';

            IniFile.WriteInteger(Section, 'Top', Top);
            IniFile.WriteInteger(Section, 'Left', Left);
         end;

         if fpSize in Remember then
         begin
            Section := ClassName + '.Window';

            IniFile.WriteInteger(Section, 'Height', Height);
            IniFile.WriteInteger(Section, 'Width', Width);
         end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

procedure TMabFormSave.Load;
var
   PName, PMesg: PChar;
   Section: string;
   AStyle: TFontStyles;
begin
   try
      if IniFile = nil then
         raise Exception.Create('Filename must be have a value');
      if not (Owner is TForm) then
         raise Exception.Create('Owner must be of type TForm');

      with Owner as TForm do
      begin
         if fpColor in Remember then
         begin
            Section := ClassName + '.Color';

            Color := StringToColor(IniFile.ReadString(Section, 'Color',
                     ColorToString(Color)));
         end;

         if fpFont in Remember then
         begin
            Section := ClassName + '.Font';

            Font.Name  := IniFile.ReadString(Section, 'Name', Font.Name);
            Font.Size  := IniFile.ReadInteger(Section, 'Size', Font.Size);
            Font.Color := StringToColor(IniFile.ReadString(Section, 'Color',
                          ColorToString(Font.Color)));

            AStyle := [];
            if IniFile.ReadBool(Section, 'Italic', fsItalic in Font.Style) then
               AStyle := AStyle + [fsItalic];

            if IniFile.ReadBool(Section, 'Bold', fsBold in Font.Style) then
               AStyle := AStyle + [fsBold];

            if IniFile.ReadBool(Section, 'Underline', fsUnderline in Font.Style) then
               AStyle := AStyle + [fsUnderline];

            if IniFile.ReadBool(Section, 'Strikeout', fsStrikeout in Font.Style) then
               AStyle := AStyle + [fsStrikeout];

            Font.Style := AStyle;
         end;

         WindowState := wsNormal;
         if fpPosition in Remember then
         begin
            Section := ClassName + '.Window';
            Top := IniFile.ReadInteger(Section, 'Top', Top);
            Left := IniFile.ReadInteger(Section, 'Left', Left);
         end;

         if fpSize in Remember then
         begin
            Section := ClassName + '.Window';
            Height := IniFile.ReadInteger(Section, 'Height', Height);
            Width := IniFile.ReadInteger(Section, 'Width', Width);
         end;

         if fpWindowState in Remember then
         begin
            Section := ClassName + '.Window';

            WindowState := wsNormal;
            if IniFile.ReadBool(Section, 'Maximized', False) then
               WindowState := wsMaximized;

            if IniFile.ReadBool(Section, 'Minimized', False) then
               WindowState := wsMinimized;
         end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

end.
